#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>

#define F_CPU 16000000
#include <util/delay.h>

#include "main.h"

int foundSpace = 0;

int main()
{
	//initializare
	initSystem();
	initADC(); // initializare ADC
	
	while(1)
	{
		//initializare
		initSystem();
		initADC(); // initializare ADC
		while(!goPark()) // a fost apasat butonul de gasire loc pentru parcare?
		{
			findSpace(); // caut loc liber
			//_delay_ms(2000);
			park(); // execut manevrele de parcare
		}
	}
	return 0;
}

void initSystem()
{
	//setare pinii din PORTB ca fiind iesire pentru cele 4 LED-uri care arata directia de mers
	LED_PORT_DIR = 0xFF; //(1<<LED_FORWARD) | (1<<LED_LEFT) | (1<<LED_RIGHT) | (1<<LED_BACKWARDS);
	LED_PORT = 0x00;
	
	//setare pin iesire pentru LED-ul de stare...
	DDRD = (1<<PD7);
	PORTD = (1<<PD7);
	
	//setare pini pentru motorase
	ENGINES_PORT_DIR = 0xFF; //(1<<STEERING_ENGINE_PIN1) | (1<<STEERING_ENGINE_PIN2) | (1<<DRIVING_ENGINE_PIN1) | (1<<DRIVING_ENGINE_PIN2);
	ENGINES_PORT = 0x00;

	// initializare buton pentru pornire a masinutei
	DDRD &= ~(1<<PD6);
	PORTD |= (1<<PD6);
	
	// nu avem loc disponibil de parcare
	foundSpace = 0;
}

void initADC() // initializarea ADC-ului
{
	ADMUX = (1<<REFS0); // sursa: AVCC cu condesator extern
	ADCSRA = (1<<ADEN); // activez ADC-ul
	ADCSRA |= (1<<ADPS2) | (1<<ADPS1) | (ADPS0); // prescaler 128;
}

unsigned int readADCDistance(unsigned char channel) // citirea distantei curente dupa conversia ADC-ului
{
	ADMUX |= channel; //setez canalul
	ADCSRA |= (1<<ADSC); // pornesc conversia
	while ( !(ADCSRA & (1<<ADIF)) ); // astept sa se termine conversia
	return ADC;
}

int goPark()
{
	while(1)
		if((PIND & (1<<PD6)) == 0)
			return 0;
	return 1;
}	

void lightUpLEDs(int led1, int led2)
{
	LED_PORT = 0;
	if(led1 != NONE)
		LED_PORT = (1<<led1);
	if(led2 != NONE)
		LED_PORT |= (1<<led2);
}

void modifyLEDsStatus()
{
	LED_PORT = LED_PORT ^ ((1<<LED_FORWARD) | (1<<LED_BACKWARDS) | (1<<LED_LEFT) | (1<<LED_RIGHT));
}

void findSpace()
{
	foundSpace = 0;
	lightUpLEDs(LED_FORWARD, NONE);
	//merg inainte pentru a gasi un loc
	driveForward();
	while(!foundSpace)
		search(); // analizez datele de la senzorul de distanta
	// mai merg putin in fata pentru a ma pozitiona pentru manevra de parcare
	_delay_ms(500);
	stopCar();
	// am gasit loc de parcare si notific vizual acest lucru:) 
	LED_PORT = (1<<LED_FORWARD) | (1<<LED_BACKWARDS) | (1<<LED_LEFT) | (1<<LED_RIGHT);
	int i;
	_delay_ms(1000);
	
	for(i=0; i<4; i++)
	{
		modifyLEDsStatus();
		_delay_ms(1000);
	}
}

void search()
{
	// aici voi folosi senzorul de distanta pentru a cauta locul liber pentru parcare
	int time = 0;
	unsigned int currentDistance; // distanta pana la cel mai apropiat obiect din dreapta masinii
	unsigned int distanceNeeded = 35; // 14 inch
	int timeToSleep = 50;
	int totalTime = 1000;
	int i, sum = 0;
	unsigned int nTries = 5;
	unsigned int refreshRate = 50;
	while(time < totalTime) // timpul in care masinuta parcurge lungimea necesara unui loc de parcare
	{
	
		sum = 0;
		for(i=0; i<nTries; i++) // realizez 5 masuratori
		{
			currentDistance = (readADCDistance(0)/2) * 2.54; // citesc distanta prin intermediul ADC-ului si o transform in cm
			sum += currentDistance;
			_delay_ms(refreshRate); // rata la care poate face senzorul citirile
		}
		currentDistance = sum/nTries; // fac media masuratorilor pentru o mai buna aproximare a distantei
		if(currentDistance >= distanceNeeded) 
		{
			_delay_ms(timeToSleep);
			time += timeToSleep + nTries*refreshRate;
			PORTD |= (1<<PD7); // aprind LED-ul de stare
		} else
		{
			time = 0;
			PORTD &= ~(1<<PD7); // sting LED-ul de stare
		}
	}
	foundSpace = 1;
}

void park()
{
	// secventa miscarilor de parcare 
	stopCar();
	
	lightUpLEDs(NONE, LED_RIGHT);
	turnRight();
	_delay_ms(1000);
	lightUpLEDs(LED_BACKWARDS, LED_RIGHT);
	driveBackwards();
	_delay_ms(1000);
	
	stopCar();
	_delay_ms(1000);
	
	lightUpLEDs(NONE, LED_LEFT);
	turnLeft();
	_delay_ms(1000);
	lightUpLEDs(LED_BACKWARDS, LED_LEFT);
	driveBackwards();
	_delay_ms(750);
	
	stopCar();
	_delay_ms(1000);
	
	lightUpLEDs(LED_FORWARD, NONE);
	driveForward();
	_delay_ms(500);
	
	stopCar();
	LED_PORT = 0x00;
}

void stopCar()
{
	//resetez portul celor doua motoare
	ENGINES_PORT = 0x00;
}

void driveForward()
{
	//pornesc motorul 
	ENGINES_PORT |= (1<<DRIVING_ENGINE_PIN2);
	ENGINES_PORT &= ~(1<<DRIVING_ENGINE_PIN1);
}

void driveBackwards()
{
	//pornesc motorul
	ENGINES_PORT |= (1<<DRIVING_ENGINE_PIN1);
	ENGINES_PORT &= ~(1<<DRIVING_ENGINE_PIN2);
}

void turnRight()
{
	//pornesc motorul
	ENGINES_PORT |= (1<<STEERING_ENGINE_PIN1);
	ENGINES_PORT &= ~(1<<STEERING_ENGINE_PIN2);
}

void turnLeft()
{
	//pornesc motorul
	ENGINES_PORT |= (1<<STEERING_ENGINE_PIN2);
	ENGINES_PORT &= ~(1<<STEERING_ENGINE_PIN1);
}